unit clan;
{ Cluster Analysis

  Copyright (C) 2013 Vasily I. Volchenko daesher@ya.ru

  This library is free software; you can redistribute it and/or modify it
  under the terms of the GNU Library General Public License as published by
  the Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public License
  for more details.

  You should have received a copy of the GNU Library General Public License
  along with this library; if not, write to the Free Software Foundation,
  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
}
{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils,matrices,contnrs,math,clanutils,Dialogs;
type

 { TClusterElement }

  TClusterElement=class(TVector)
  private
    FNumber: integer;
  public
    function Distance(E:TClusterElement):Double;
    function Copy:TClusterElement;
    property Number:integer read FNumber write FNumber;
    constructor CreateFromMatrix(M:TMatrix;Row:integer);virtual;
  end;
  { TCluster }
  TCluster=class(TObject)
  private
    FContainer: boolean;
    FElements:TObjectList;
    function GetCoord(N: integer): Double;
    function GetCoordCount: integer;
    function GetElemCount: integer;
    function GetElements(N: integer): TClusterElement;
    procedure SetElements(N: integer; AValue: TClusterElement);
  public
    property Container:boolean read FContainer;
    constructor Create; virtual;
    constructor Create(Element:TClusterElement);virtual;
    property ElemCount:integer read GetElemCount;
    property CoordCount:integer read GetCoordCount;
    property Coord[N:integer]:Double read GetCoord;
    property Elements[N:integer]:TClusterElement read GetElements write SetElements;
    procedure AddCluster(Cl:TCluster;FreeIt:boolean=false);
    procedure AddElement(E:TClusterElement);
    function MoveFrom(Cl:TCluster;Idx:integer):boolean;
    function Distance(Cl:TCluster):Double;
    function Distance(Cl:TClusterElement):Double;
    procedure Remove(E:TClusterElement);
    destructor Destroy; override;
  end;

function ClusterAnalisys(M:TMatrix;N:integer):TVector;

var Progresser:procedure(N:integer;Dist:double) of object = nil;
    UseKMean:boolean=false;
    UseForEl:boolean=false;
    ForElR:double=10.0;
implementation

function ClusterAnalisys(M: TMatrix;N:integer): TVector;
var
  OL : TObjectList;
  i,j: Integer;
  E  , E1: TClusterElement;
  MD:record
    Dist:Double;
    N1,N2:integer;
  end;
  Cl, Cl1, Cl2: TCluster;
  Dst: Double;
  Step, k:integer;
  NC1: LongInt;
  NC2: LongInt;
  NE1: LongInt;
  First: Boolean;
  function KMDist:Double;
  var   S1,S2: Double;
  i,j,k: Integer;
  L: Integer;
  Cl:TCluster;
  begin
    S1:=0;
    for i:=0 to OL.Count-1 do
    begin
      S2:=0;L:=0;
      Cl:=OL[i] as TCluster;
      if Cl.ElemCount=0 then Continue;
      for j:=0 to Cl.ElemCount-1 do
      begin
        S2:=S2+sqr(Cl.Distance(Cl.Elements[j]));
        inc(L);
      end;
        {for k:=j+1 to Cl.ElemCount-1 do
        begin
          S2:=S2+sqr(Cl.Distance()
          //Cl.Elements[j].Distance(Cl.Elements[k]));
          inc(L);
        end;}
      if L<>0 then S1:=S1+S2/L;
      if L<2 then S1:=S1+30;//Не даём создавать пустых и единичных кластеров без необх.
    end;
    Result:=S1{/OL.Count};
  end;

begin
  Result:=nil;
  OL:=TObjectList.create(true);
  if UseKMean then
  begin
    //1.Создаём пустые кластеры и заполняем их случайно
    for i:=1 to N do
      OL.Add(TCluster.Create);
    Randomize;
    for i:=0 to M.RowCount-1 do
    begin
      E:=TClusterElement.CreateFromMatrix(M,i);
      E.Number:=i;
      Cl:=OL[Random(N)] as TCluster;
      Cl.AddElement(E);
    end;
    Step:=0;
    repeat
      //2. Проверяем расстояние между всеми элементами в каждом кластере
      MD.Dist:=KMDist;
      //3. Создаём план замены
      repeat
        NC1:=Random(OL.Count);// Из кластера
      until (OL[NC1] as TCluster).ElemCount>0;
      repeat
        NC2:=Random(OL.Count);// В кластер
      until NC2<>NC1;
      Cl:=(OL[NC1] as TCluster);
      NE1:=Random(Cl.ElemCount);
      E:=Cl.Elements[NE1];
      Cl.Remove(E);
      Cl1:=(OL[NC2] as TCluster);
      Cl1.AddElement(E);
      Dst:=KMDist;
      if MD.Dist<=Dst then begin
        //Вернуть на место!!!
        Cl1.Remove(E);
        Cl.AddElement(E);
        if KMDist>(MD.Dist+EPS) then ShowMessage('Ошибка: дистанция была '
        +FloatToStr(MD.Dist)+' стала после возврата '+ FloatToStr(KMDist));
        inc(Step);
      end;// else if (Dst-MD.Dist)>EPS then ;
      if (MD.Dist-Dst)<EPS then inc(Step) else Step:=Step -1;
      if Assigned(Progresser) then Progresser(Step,MD.Dist);
    until Step>M.RowCount*100;
  end
  else if UseForEl then
  begin
    OL:=TObjectList.create(true);
    Cl:=TCluster.Create;//Кластер со всеми неиспользованными элементами
    for i:=0 to M.RowCount-1 do
    begin
      E:=TClusterElement.CreateFromMatrix(M,i);
      E.Number:=i;
      Cl.AddElement(E);
    end;
    Randomize;
    repeat
      E:=(Cl.Elements[Random(Cl.ElemCount)]).Copy;
      E1:=e.Copy;
      First:=true;
      k:=0;
      repeat
        if not First then Cl.AddCluster(Cl1,true)
           else First:=false;
        Cl1:=TCluster.Create;
        for i:=0 to cl.ElemCount-1 do
          if i<Cl.ElemCount then //Hack
            if E1.Distance(Cl.Elements[i])<=ForElR then
              Cl1.MoveFrom(Cl,i);
        E.Free;
        E:=E1;
        E1:=TClusterElement.Create;
        if Cl1.ElemCount=0 then break;
        for i:=0 to Cl1.CoordCount-1 do
          E1[i]:=Cl1.Coord[i];
        E1.Number:=-1;
        if Assigned(Progresser) then Progresser(OL.Count,E1.Distance(E));
        inc(k);
      until (E1.Distance(E)<=Eps)or (k>100);
      E.Free;
      E1.Free;
      OL.Add(Cl1);
    until Cl.ElemCount=0;
  end
  else  //Иерархический метод
  begin
    for i:=0 to M.RowCount-1 do
    begin
      E:=TClusterElement.CreateFromMatrix(M,i);
      E.Number:=i;
      Cl:=TCluster.Create(E);
      OL.Add(Cl);
    end;
    //Создали кластеры. Теперь попробуем объединить
    if OL.Count=0 then exit;
    Step:=0;

    while OL.Count>N do
    begin
      inc(Step);
      MD.Dist:=MaxDouble;
      if Assigned(Progresser) then Progresser(Step,MD.Dist);
      for i:=0 to DivMore(OL.Count,2)-1 do     //если 0, то будет -1 - делать нечего
        for j:=0 to OL.Count-1 do
        begin
          if i=j then continue;
          Dst:=(OL[i] as TCluster).Distance(OL[j] as TCluster);
          if Dst<MD.Dist then begin
            MD.Dist:=Dst;
            MD.N1:=i;
            MD.N2:=j;
          end;
          if Assigned(Progresser) then Progresser(Step,MD.Dist);
        end;
      if MD.Dist=MaxDouble then raise EMathError.Create('Не нашли минимальное расстояние');
      Cl:=(OL[MD.N1] as TCluster);
      Cl.AddCluster(OL[MD.N2] as TCluster);
      OL.Delete(MD.N2);
    end;

  end;
  //Кластеров осталось сколько надо. Выводим вектор
  Result:=TVector.create;
  for i:=0 to OL.Count-1 do
    for j:=0 to (OL[i] as TCluster).ElemCount-1 do
      Result[(OL[i] as TCluster).Elements[j].Number]:=i+1;
  OL.Free;
end;

{ TCluster }
function TCluster.GetElemCount: integer;
begin
  Result:=FElements.Count;
end;

function TCluster.GetCoord(N: integer): Double;
var
  i,N1: Integer;
  s: Double;

begin
  s:=0.0;N1:=0;
  for i:=0 to ElemCount do
    if Elements[i]<>nil then
    begin
      S:=S+Elements[i].Values[N];
      inc(N1);
    end;
  Result:=S/N1;
end;

function TCluster.GetCoordCount: integer;
var
  i: Integer;
begin
  Result:=0;
  for i:=0 to ElemCount-1 do
    if Elements[i].Count>Result then Result:=Elements[i].Count;
end;

function TCluster.GetElements(N: integer): TClusterElement;
begin
  Result:=nil;
  if (N>=ElemCount) or (N<0) then exit;
  if not (FElements[N] is TClusterElement) then exit;
  Result:=FElements[N] as TClusterElement;
end;


procedure TCluster.SetElements(N: integer; AValue: TClusterElement);
begin
  While ElemCount<=N do FElements.Add(nil);
  FElements[N]:=AValue;
end;

constructor TCluster.Create;
begin
  inherited Create;
  FContainer:=false;
  FElements:=TObjectList.create(false);
end;

constructor TCluster.Create(Element: TClusterElement);
begin
  Create;
  FElements.Add(Element);
end;

procedure TCluster.AddCluster(Cl: TCluster; FreeIt: boolean);
var
  i: Integer;
begin
  for i:=0 to Cl.ElemCount-1 do
    if Cl.Elements[i]<>nil then FElements.Add(Cl.Elements[i]);
  while Cl.ElemCount>0 do begin
    Cl.Elements[0]:=nil;
    Cl.FElements.Delete(0);
  end;
  if FreeIt then Cl.Free;
end;

procedure TCluster.AddElement(E: TClusterElement);
begin
  FElements.Add(E);
end;

function TCluster.MoveFrom(Cl: TCluster; Idx: integer):boolean;
var
  E: TClusterElement;
begin
  Result:=false;
  if Cl.ElemCount<=Idx then exit;
  E:=Cl.Elements[Idx];
  if E=Nil then exit;
  AddElement(E);
  Cl.Remove(E);
end;

function TCluster.Distance(Cl: TCluster): Double;
var
  i: Integer;
  N: Integer;
begin
  Result:=0;
  N:=CoordCount-1;
  for i:=0 to N do
    Result:=Result+sqr(self.Coord[i]-cl.Coord[i]);
  Result:={sqrt}(Result/ElemCount)
end;

function TCluster.Distance(Cl: TClusterElement): Double;
var
  i: Integer;
  N: Integer;
begin
  Result:=0;
  N:=CoordCount-1;
  for i:=0 to N do
    Result:=Result+sqr(self.Coord[i]-cl.Values[i]);
  Result:={sqrt}(Result/ElemCount)

end;

procedure TCluster.Remove(E: TClusterElement);
begin
  FElements.Remove(E);
end;

destructor TCluster.Destroy;
var
  i: Integer;
  E: TClusterElement;
begin
  for i:=0 to ElemCount do
   if Elements[i]<>nil then begin
     E:=Elements[i];
     FElements.Delete(i);
     E.Free;
   end;
  inherited Destroy;
end;

{ TClusterElement }

function TClusterElement.Distance(E: TClusterElement): Double;
var
  i: Integer;
begin
  Result:=0;
  for i:=0 to Count-1 do
    Result:=Result+sqr(self[i]-E[i]);
  Result:=sqrt(Result/Count);
end;

function TClusterElement.Copy: TClusterElement;
var
  i: Integer;
begin
  Result:=TClusterElement.create;
  Result.Number:=-1;
  for i:=0 to Count-1 do
    Result[i]:=Self[i];
  if Result.Count<Self.Count then raise EInOutError.Create('Неверное количество');
end;

constructor TClusterElement.CreateFromMatrix(M: TMatrix; Row: integer);
var
  i: Integer;
begin
  Create;
  for i:=0 to M.ColCount-1 do
    Self[i]:=M[i,Row];
end;

end.

