unit clanu1;
{ Cluster Analysis

  Copyright (C) 2013 Vasily I. Volchenko daesher@ya.ru

  This library is free software; you can redistribute it and/or modify it
  under the terms of the GNU Library General Public License as published by
  the Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public License
  for more details.

  You should have received a copy of the GNU Library General Public License
  along with this library; if not, write to the Free Software Foundation,
  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
}
// Программа кластерного анализа простейшим иерархическим методом
// Рассматривается как часть проекта vvfstat. Возможно, будет объединена с самой
// программой vvfstat
// Вы можете скачать актуальную версию на сайте vvfstat.sf.net

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, Grids, Menus,
  clanutils,local,matrices,clan;

type

  { TClusterAnForm }

  TClusterAnForm = class(TForm)
    MainMenu1: TMainMenu;
    mColNum: TMenuItem;
    mAnalysis: TMenuItem;
    mClusterAnalys: TMenuItem;
    MenuItem1: TMenuItem;
    MenuItem2: TMenuItem;
    MenuItem3: TMenuItem;
    MenuItem4: TMenuItem;
    mUseForEl: TMenuItem;
    mUse_k_Mean: TMenuItem;
    mExit: TMenuItem;
    mLoad: TMenuItem;
    mSaveAs: TMenuItem;
    mNumRows: TMenuItem;
    mTable: TMenuItem;
    mFile: TMenuItem;
    ODlg: TOpenDialog;
    SvDlg: TSaveDialog;
    sgTable: TStringGrid;
    procedure mClusterAnalysClick(Sender: TObject);
    procedure mColNumClick(Sender: TObject);
    procedure MenuItem3Click(Sender: TObject);
    procedure mExitClick(Sender: TObject);
    procedure mLoadClick(Sender: TObject);
    procedure mNumRowsClick(Sender: TObject);
    procedure mSaveAsClick(Sender: TObject);
  private
    { private declarations }
  public
    procedure Prgr(N:integer;Dist:double);
  end;

var
  ClusterAnForm: TClusterAnForm;

implementation

{$R *.lfm}

{ TClusterAnForm }

procedure TClusterAnForm.mNumRowsClick(Sender: TObject);
var
  N: Integer;
  i: Integer;
begin
  N:=InputInteger(rsEnterRowNum, '', 10, cd_Natural);
  sgTable.RowCount:=N+1;
  for i:=1 to N do
    sgTable.Cells[0,i]:=IntToStr(i);
end;

procedure TClusterAnForm.mSaveAsClick(Sender: TObject);
var
  FS:TFileStream;
  i: Integer;
  j: Integer;
begin
  if not SvDlg.Execute then exit;
  try
    FS:=TFileStream.Create(SvDlg.FileName,fmCreate);
  except
    ShowMessage('Ошибка создания файла');
    exit;
  end;
  try
    FS.Write(Signature,SizeOf(Signature));
    FS.WriteWord(sgTable.ColCount);
    FS.WriteWord(sgTable.RowCount);
    for i:=0 to sgTable.ColCount-1 do
      for j:=0 to sgTable.RowCount-1 do
        FS.WriteAnsiString(sgTable.Cells[i,j]);

  except
    ShowMessage('Ошибка записи в файл');
  end;
  FS.Free;
end;



procedure TClusterAnForm.Prgr(N: integer; Dist: double);
begin
  Caption:='Шаг '+IntToStr(N)+', расстояние '+FloatToStr(Dist);
  Application.ProcessMessages;
end;

procedure TClusterAnForm.mColNumClick(Sender: TObject);
var
  N: Integer;
  i: Integer;
begin
  N:=InputInteger('Введите число столбцов','',5,cd_Natural);
  sgTable.ColCount:=N+1;
  for i:=1 to N do
    sgTable.Cells[i,0]:=IntToStr(i);
end;

procedure TClusterAnForm.MenuItem3Click(Sender: TObject);
begin
  ShowMessage('Программа кластерного анализа иерархическим методом.'+LineEnding+
     'Лицензия - LGPL. Версия - '+VerString+LineEnding+'Главный разработчик - '+
     PrjMaintainer);
end;

procedure TClusterAnForm.mClusterAnalysClick(Sender: TObject);
var
  Sel,Sel1: TGridRect;
  Wrong: Boolean;
  Err: Boolean;
  M: TMatrix;
  i: LongInt;
  j: LongInt;
  F: Double;
  NGr: Integer;
  V: TVector;
begin
  Sel:=sgTable.Selection;
  UseKMean:=mUse_k_Mean.Checked;
  UseForEl:=mUseForEl.Checked;
  if UseForEl then
    ForElR:=InputReal('Введите радиус R','R=',10);
  Wrong:=false;
  Progresser:=@Prgr;
  if Sel.Right-Sel.Left<0 then Wrong:=true;
  if Sel.Bottom-Sel.Top<3 then Wrong:=true;
  if Wrong then Begin
    ShowMessage('Диапазон обработки не выбран или очень мал');
    exit;
  end;
  if Sel.Right=sgTable.ColCount-1 then //Добавляем ещё одну колонку для заполнения
  begin
    sgTable.ColCount:=sgTable.ColCount+1;
    sgTable.Cells[sgTable.ColCount-1,0]:=IntToStr(sgTable.ColCount-1);
  end;
  M:=TMatrix.Create;

  for i:=Sel.Left to Sel.Right do
    for j:=Sel.Top to Sel.Bottom do
    begin
      Err:=false;
      F:=AnyStrToFloat(sgTable.Cells[i,j],Err);
      if Err then begin
        ShowMessage('Обнаружены нечисловые данные!!!');
        Sel1.Left:=i;
        Sel1.Right:=i;
        Sel1.top:=j;
        Sel1.Bottom:=j;
        sgTable.Selection:=Sel1;
        exit;
      end;
      M[i-Sel.Left,j-Sel.Top]:=F;
    end;
  if not UseForEl then
     NGr:=InputInteger('Введите ориентировочное число классов','',3,cd_Natural);
  V:=ClusterAnalisys(M,NGr);
  for j:=Sel.Top to Sel.Bottom do
    sgTable.Cells[Sel.Right+1,j]:=IntToStr(round(V[j-Sel.Top]));
end;

procedure TClusterAnForm.mExitClick(Sender: TObject);
begin
  Close;
end;

procedure TClusterAnForm.mLoadClick(Sender: TObject);
var
  FS: TFileStream;
  SGN:array[1..5]of char;
  w: Word;
  i: Integer;
  j: Integer;
begin
  if not ODlg.Execute then exit;
  if not FileExistsUTF8(ODlg.FileName) then
  begin
    ShowMessage('Файл '+ODlg.FileName+' не существует!');
    exit;
  end;
  try
    FS:=TFileStream.Create(ODlg.FileName,fmOpenRead);
  except
    ShowMessage('Ошибка открытия файла');
    exit;
  end;
  try
    FS.Read(SGN,sizeof(Signature));
    if SGN<>Signature then
    begin
      ShowMessage('Неверная структура файла');
      FS.Free;
      exit;
    end;
    w:=FS.ReadWord;
    sgTable.ColCount:=w;
    w:=FS.ReadWord;
    sgTable.RowCount:=w;
    for i:=0 to sgTable.ColCount-1 do
      for j:=0 to sgTable.RowCount-1 do
        sgTable.Cells[i,j]:=FS.ReadAnsiString;
  except
    ShowMessage('Ошибка чтения из файла');
  end;
  FS.Free;
end;

end.

