unit clanutils;
{ Cluster Analysis

  Copyright (C) 2013 Vasily I. Volchenko daesher@ya.ru

  This library is free software; you can redistribute it and/or modify it
  under the terms of the GNU Library General Public License as published by
  the Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public License
  for more details.

  You should have received a copy of the GNU Library General Public License
  along with this library; if not, write to the Free Software Foundation,
  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
}
{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Dialogs, Local;
Const Eps=1e-5;
type TCondition=(cd_None,cd_NonNeg,cd_Natural,cd_Interval);
function InputInteger(const Cap,Prompt:string;Default:integer=0;
    Condition:TCondition=cd_None;AFrom:integer=0;ATo:Integer=0):integer;
function InputReal(const Cap,Prompt:string;Default:double=0;
    Condition:TCondition=cd_None;AFrom:double=0;ATo:double=0):double;
function AnyStrToFloat(s:string;out Error:Boolean):Double;
function DivMore(a,b:integer):integer;

implementation

function DivMore(a,b:integer):integer;
begin
  if a mod b =0 then Result:=a div b else Result:=(a div b)+1;
end;

function InputInteger(const Cap, Prompt: string; Default: integer;
  Condition: TCondition; AFrom: integer; ATo: Integer): integer;
var
  s,APrompt: String;
  Code:integer;
begin
  if Prompt='' then APrompt:=Cap else APrompt:=Prompt;
  repeat
    s:=InputBox(Cap,APrompt,IntToStr(Default));
    Val(S,Result,Code);
    if Code<>0 then APrompt:=rsNonInt+Prompt;
    if (Condition=cd_NonNeg) and (Result<0) then
      begin
        APrompt:=rsNoNonNeg+Prompt;
        Code:=1;
      end;
    if (Condition=cd_Natural) and (Result<1) then
      begin
        APrompt:=rsNotNat+Prompt;
        Code:=1;
      end;

      if (Condition=cd_Interval) and ((Result<AFrom)or(Result>Ato)) then
      begin
        APrompt:=rsNoInterval+Prompt;
        Code:=1;
      end;
  until Code=0;
end;

function InputReal(const Cap, Prompt: string; Default: double;
  Condition: TCondition; AFrom: double; ATo: double): double;
var
  s,APrompt: String;
  Code:integer;
  ODC:char;
  p: Integer;
begin
  if Prompt='' then APrompt:=Cap else APrompt:=Prompt;
  repeat
    ODC:=DecimalSeparator;
    DecimalSeparator:='.';
    s:=InputBox(Cap,APrompt,FloatToStr(Default));
    repeat
      p:=pos(',',s);
      if p>0 then s[p]:='.';
    until p<=0;
    Val(S,Result,Code);
    DecimalSeparator:=ODC;
    if Code<>0 then APrompt:=rsNonInt+Prompt;
    if (Condition=cd_NonNeg) and (Result<0) then
      begin
        APrompt:=rsNoNonNeg+Prompt;
        Code:=1;
      end;
    if (Condition=cd_Natural) and (Result<1) and (frac(abs(Result))>Eps) then
      begin
        APrompt:=rsNotNat+Prompt;
        Code:=1;
      end;

      if (Condition=cd_Interval) and ((Result<AFrom)or(Result>Ato)) then
      begin
        APrompt:=rsNoInterval+Prompt;
        Code:=1;
      end;
  until Code=0;
end;

function AnyStrToFloat(s: string; out Error: Boolean): Double;
var
  ODS: Char;
begin
  Error:=false;
  ODS:=DecimalSeparator;
  DecimalSeparator:=',';
  try
   Result:=StrToFloat(s);
  except
    try
      DecimalSeparator:='.';
      Result:=StrToFloat(s);
    except
      Error:=true;
    end;
  end;
  DecimalSeparator:=ODS;
end;


end.

