﻿unit matrices;
{$ifdef fpc}
{$mode objfpc}
{$endif}
{$H+}
{$ifndef fpc}
{$ifndef VIRTUALPASCAL}
{$ifndef delphi}
{$define pabc}
{$endif}
{$endif}
{$endif}
interface

uses
  Contnrs, Classes, SysUtils,math,Sets;
type

 TMatrixItem=class(TObject)
 public
  Holder:extended;
 end;

 { TVector }

 TVector=class(TObject)
 private
   function GetValues(i: integer): Extended;
   procedure SetValues(i: integer; const AValue: Extended);
 protected
  FRootList:TObjectList;
 public
  constructor create;{$ifndef pabc}virtual;{$endif}
  function Count:integer;virtual;
  destructor Destroy;{$ifdef pabc}virtual{$else}override{$endif};
  function Abs:Extended;virtual;
  procedure Assign(Vector:TVector);virtual;
  property Values[i:integer]:Extended read GetValues write SetValues;default;
 end;

 { TMatrix }

 TMatrix=class(TObject)
 private
   function GetColCount: integer;
   function GetRowCount: integer;
   function GetValues(i, j: integer): extended;
   procedure SetValues(i, j: integer; const AValue: extended);
 protected
  FRootList:TObjectList;
 public
  constructor Create;{$ifndef pabc}virtual;{$endif}
  property ColCount:integer read GetColCount;
  property RowCount:integer read GetRowCount;
  property Values[i,j:integer]:extended read GetValues write SetValues;default;
  destructor destroy;{$ifdef pabc}virtual{$else}override{$endif};
  procedure Xk(k:extended);virtual;
 end;
//В основном, матричные операции будут представлены функциями. Операторы - доп.

{ TErrorousMatrix }

TErrorousMatrix=class(TMatrix)
public
  Reason:string;
  constructor Create(const AReason:string);
end;

function MatMul(const M1:TMatrix;M2:TMatrix):TMatrix;
function Transponate(const M:TMatrix):TMatrix;
function Det(const M:TMatrix):extended;
function Minor(const M:TMatrix;x,y:integer):TMatrix;
function MatrixC(const M:TMatrix):TMatrix;
function InverseMatrix(const M:TMatrix):TMatrix;

var ObjectCache:TObjectCache;
    Caching:boolean{$ifdef pabc}:={$else}={$endif}true;
implementation

function MatMul(const M1: TMatrix; M2: TMatrix): TMatrix;
var
  j: Integer;
  i: Integer;
  Sum0: Extended;
  k: Integer;
begin
  if (Not Assigned(M1))or(Not Assigned(M2)) then begin
    Result:=TErrorousMatrix.Create('Argument(s) for multiply is(are) not assigned');
    if Caching then ObjectCache.Add(Result);
    exit;
  end;
  if M1.RowCount<>M2.ColCount then begin
    Result:=TErrorousMatrix.Create('Matrices are not fited for multiply');
    if caching then ObjectCache.Add(Result);
    exit;
  end;
  Result:=TMatrix.Create;
  if Caching then if caching then ObjectCache.Add(Result);
  Result[M1.ColCount-1,M2.RowCount-1]:=0.0;
  for i:=0 to Result.ColCount-1 do
    for j:=0 to Result.RowCount-1 do
    begin
      Sum0:=0.0;
      for k:=0 to M1.RowCount-1 do
        Sum0:=Sum0+M1[i,k]*M2[k,j];
      Result[i,j]:=Sum0;
    end;
end;

function Transponate(const M: TMatrix): TMatrix;
var
  i: Integer;
  j: Integer;
begin
  Result:=TMatrix.Create;
  for i:=0 to M.ColCount-1 do
    for j:=0 to M.RowCount-1 do
      Result[j,i]:=M[i,j];
end;
function Minus1PWR(i:integer):integer;
begin
  if (i mod 2)=0 then Result:=1 else Result:=-1;
end;

function Det(const M: TMatrix): extended;
var
  OldCaching: Boolean;
  N: LongInt;
  i: Integer;
  MNR: TMatrix;
begin
  if (not Assigned(M))or(m.ColCount<>m.RowCount) then begin Result:=NaN; exit;end;
  OldCaching:=Caching;
  N:=M.ColCount;
  if N=2 then
  begin
    Result:=M[0,0]*M[1,1]-M[1,0]*M[0,1];
    exit;
  end;
  Caching:=false;
  Result:=0;
  for i:=0 to N-1 do
  begin
    MNR:=Minor(M,0,i);
    Result:=Result+M[0,i]*Det(MNR)*Minus1PWR(i);//i+1+1
    MNR.Free;
  end;
  Caching:=OldCaching;
end;

function Minor(const M: TMatrix; x, y: integer): TMatrix;
var
  i: Integer;
  i1: LongInt;
  j: Integer;
  j1: LongInt;
begin
  if (not Assigned(M))or(M.ColCount<=x)or(M.RowCount<=y) then
  begin
    Result:=TErrorousMatrix.Create('Wrong data for minor matrix');
    if Caching then ObjectCache.Add(M);
    exit;
  end;
  Result:=TMatrix.Create;
  if Caching then ObjectCache.Add(Result);
  for i:=0 to M.ColCount-1 do
    for j:=0 to M.RowCount-1 do
    begin
      if (i=x)or(j=y) then continue;//todo don''t waste a time
      if i<x then i1:=i else i1:=i-1;
      if j<y then j1:=j else j1:=j-1;
      Result[i1,j1]:=M[i,j];
    end;
end;

function MatrixC(const M: TMatrix): TMatrix;
var
  OldCaching: Boolean;
  i: Integer;
  j: Integer;
  MNR: TMatrix;
begin
  if (not Assigned(M))or(M.ColCount<>M.RowCount) then
  begin
    Result:=TErrorousMatrix.Create('Wrong data for minor matrix');
    if Caching then ObjectCache.Add(M);
    exit;
  end;
  Result:=TMatrix.Create;
  if Caching then ObjectCache.Add(M);
  OldCaching:=Caching;
  for i:=0 to M.ColCount-1 do
    for j:=0 to M.RowCount-1 do
    begin
      MNR:=Minor(M,i,j);
      Result[i,j]:=Minus1PWR(i+j)*det(MNR);//i+1+j+1
      MNR.Free;
    end;
  Caching:=OldCaching;
end;

function InverseMatrix(const M: TMatrix): TMatrix;
var
  OldCaching: Boolean;
  M1: TMatrix;
  detM: Extended;
begin
  if (not Assigned(M))or(M.ColCount<>M.RowCount) then
  begin
    Result:=TErrorousMatrix.Create('Wrong data for minor matrix');
    if Caching then ObjectCache.Add(M);
    exit;
  end;
  if Caching then ObjectCache.Add(M);
  OldCaching:=Caching;
  M1:=MatrixC(M);
  Result:=Transponate(M1);
  M1.Free;
  if OldCaching then ObjectCache.Add(Result);
  detM:=Det(M);
  Result.Xk(1/detM);
  Caching:=OldCaching;
end;

{ TMatrix }

function TMatrix.GetColCount: integer;
begin
 Result:=FRootList.Count;
end;

function TMatrix.GetRowCount: integer;
var
  i: Integer;
begin
 Result:=0;
 if ColCount=0 then exit;
 for i:=0 to ColCount-1 do
  Result:=Max(Result,(FRootList[i] as TObjectList).Count);
end;

function TMatrix.GetValues(i, j: integer): extended;
begin
try
 Result:=0;
 if i>=FRootList.Count then exit;
 if j>=TObjectList(FRootList[i]).count then exit;
 Result:=TMatrixItem(TObjectList(FRootList[i])[j]).Holder;
except
 Result:=0;
end;
end;

procedure TMatrix.SetValues(i, j: integer; const AValue: extended);
var
  k: Integer;
begin
 for k:=FRootList.Count to i do
  FRootList.Add(TObjectList.create{$ifndef pabc}(true){$endif});
 for k:=(FRootList[i] as TObjectList).Count to j do
  (FRootList[i] as TObjectList).Add(TMatrixItem.create);
 ((FRootList[i] as TObjectList)[j] as TMatrixItem).Holder:=AValue;
end;

constructor TMatrix.Create;
begin
 inherited Create;
 FRootList:=TObjectList.create{$ifndef pabc}(true){$endif};
end;

destructor TMatrix.destroy;
begin
 FRootList.free;
 inherited destroy;
end;

procedure TMatrix.Xk(k: extended);
var
  j: Integer;
  i: Integer;
begin
  for i:=0 to ColCount-1 do
    for j:=0 to RowCount-1 do
      self[i,j]:=Self[i,j]*k;
end;

{ TVector }

function TVector.GetValues(i: integer): Extended;
begin
 Result:=0;
 if i>=FRootList.Count then exit;
 Result:=(FRootList[i] as TMatrixItem).Holder;
end;

procedure TVector.SetValues(i: integer; const AValue: Extended);
var
  k: Integer;
begin
 for k:=FRootList.Count to i do FRootList.Add(TMatrixItem.create);
 (FRootList[i] as TMatrixItem).Holder:=AValue;
end;

constructor TVector.create;
begin
 inherited Create;
 FRootList:=TObjectList.create{$ifndef pabc}(true){$endif};
end;

function TVector.Count: integer;
begin
 Result:=FRootList.Count;
end;

destructor TVector.Destroy;
begin
 FRootList.free;
 inherited Destroy;
end;

function TVector.Abs: Extended;
var
  v: Extended;
  i: Integer;
begin
  v:=0.0;
  for i:=0 to Count-1 do
    v:=v+sqr(self[i]);
  Result:=sqrt(v);
end;

procedure TVector.Assign(Vector: TVector);
var
  i: Integer;
begin
 FRootList.Clear;
 for i:=0 to Vector.Count-1 do
  Self[i]:=Vector[i];
end;

{ TErrorousMatrix }

constructor TErrorousMatrix.Create(const AReason: string);
begin
  inherited Create;
  Reason:=AReason;
end;

initialization
  ObjectCache:=TObjectCache.create{$ifndef pabc}(false){$endif};
finalization
  ObjectCache.Clear;
  ObjectCache.Free;
end.

